import glob
from math import isclose
import numpy as np
import os
import pandas as pd
from pandas import ExcelWriter
import requests
import scipy.io
import scipy.signal

################################
# OVERALL Wrapper Function
################################


def get_all_data(rootdir, path_to_raw_data_folder):
    '''Gets all raw data from the rootdir (ie 'data/') and specified folder
    (path_to_raw_data_folder), i.e. 'Source_Data' (which is within rootdir),
    and then:
    1. separates it into raw cycles and puts them in a folder
    (data/Separated_Cycles/)
    2. cleans those separated cycles and puts them in a folder
    (data/Clean_Separated_Cycles/)
    3. recombines the cleaned, separated cycles and saves those
    data sets in a folder (data/Clean_Whole_Sets/). These folders
    do not have to have existed previously.'''
    assert os.path.exists(rootdir) == 1
    if not os.path.exists(rootdir):
        print('The specified rootdir does not exist.')
    if not os.path.exists(rootdir+'Separated_Cycles/'):
        os.makedirs(rootdir+'Separated_Cycles/')
    if not os.path.exists(rootdir+'Clean_Separated_Cycles/'):
        os.makedirs(rootdir + 'Clean_Separated_Cycles/')
    if not os.path.exists(rootdir + 'Clean_Whole_Sets/'):
        os.makedirs(rootdir + 'Clean_Whole_Sets/')
    load_sep_cycles(rootdir + path_to_raw_data_folder,
                    rootdir + 'Separated_Cycles/')
    get_clean_cycles(rootdir + 'Separated_Cycles/',
                     rootdir + 'Clean_Separated_Cycles/')
    get_clean_sets(rootdir + 'Clean_Separated_Cycles/',
                   rootdir+'Clean_Whole_Sets/')
    return

############################
# Sub - Wrapper Functions
############################


def load_sep_cycles(getdata_filepath, savedata_filepath):
    """Get data from a specified filepath, separates out data into
    cycles and saves those cycles as .xlsx files in specified
    filepath (must be an existing folder)"""
    dfdict = get_data(getdata_filepath)
    assert type(dfdict) == dict
    assert os.path.exists(getdata_filepath) == 1
    assert os.path.exists(savedata_filepath) == 1
    for key in dfdict:
        all_cycles_df = dfdict[key]
        cycle_dict = sep_cycles(all_cycles_df)
        battname = key
        save_sep_cycles_xlsx(cycle_dict, battname, savedata_filepath)
    print('All data separated into cycles and saved in folder "' +
          savedata_filepath + '".')
    return


def clean_calc_sep_smooth(dataframe, windowlength, polyorder):
    """Takes one cycle dataframe, calculates dq/dv, cleans the data,
    separates out charge and discharge, and applies sav-golay filter.
    Returns two dataframes, one charge and one discharge.
    Windowlength and polyorder are for the sav-golay filter."""
    assert type(dataframe) == pd.DataFrame

    df1 = calc_dv_dqdv(dataframe)
    raw_charge = df1[df1['Current(A)'] > 0]
    raw_charge = raw_charge.reset_index(drop=True)
    # this separated out the charging data and put it in 'raw_charge'.
    # Reset the index as well.

    raw_discharge = df1[df1['Current(A)'] < 0]
    raw_discharge = raw_discharge.reset_index(drop=True)
    # this separated out the discharging data and put it in 'raw_discharge'.
    # Reset the index as well.

    clean_charge2 = drop_0_dv(raw_charge)
    clean_discharge2 = drop_0_dv(raw_discharge)
    # apply the drop_0_dv function to clean out the wonky data points,
    # especially near the voltage corresponding to the end of a cycle.

    clean_charge2 = clean_charge2.sort_values(['Voltage(V)'], ascending=True)
    clean_discharge2 = clean_discharge2.sort_values(
        ['Voltage(V)'], ascending=False)
    # sorted the values because sometimes the data points are out of order,
    # especially in the first cycle of a battery.

    cleandf2 = clean_charge2.append(clean_discharge2, ignore_index=True)
    cleandf2 = cleandf2.reset_index(drop=True)
    # appends the clean charge and discharge cycles to get a full set of
    # clean data. Reset the index as well.

    charge, discharge = sep_char_dis(cleandf2)
    # apply the sep_char_dis function to cleandf2. This is necessary
    # because the sep_char_dis function actually assigns the values of
    # dq/dv to use. The discharge cycles need dq/dv to be calculated
    # based on the discharge capacity, and the charge cycles need
    # dq/dv to be calculated based on the charge capacity. This structure
    # could probably be improved since we are separating out charge/discharge,
    # then recombining, then separating again, then recombining.

    if len(discharge) > windowlength:
        smooth_discharge = my_savgolay(discharge, windowlength, polyorder)
    else:
        discharge['Smoothed_dQ/dV'] = discharge['dQ/dV']
        smooth_discharge = discharge
    # this if statement is for when the datasets have less datapoints
    # than the windowlength given to the sav_golay filter.
    # without this if statement, the sav_golay filter throws an error
    # when given a dataset with too few points. This way, we simply
    # forego the smoothing function.
    if len(charge) > windowlength:
        smooth_charge = my_savgolay(charge, windowlength, polyorder)
    else:
        charge['Smoothed_dQ/dV'] = charge['dQ/dV']
        smooth_charge = charge
    # same as above, but for charging cycles.
    return smooth_charge, smooth_discharge


def get_clean_cycles(import_filepath, save_filepath):
    """Imports all separated out cycles in given path and cleans them
    and saves them in the specified filepath"""
    assert os.path.exists(import_filepath) == 1
    assert os.path.exists(save_filepath) == 1
    rootdir = import_filepath
    # iterate through dir to get excel files
    file_list = [f for f in glob.glob(os.path.join(rootdir, '*.xlsx'))]
    d = {}  # initiate dict for data storage
    count = 0
    for file in file_list:
        count += 1
        name = os.path.split(file)[1].split('.')[0]
        data = pd.read_excel(file)
        charge, discharge = clean_calc_sep_smooth(data, 9, 3)
        clean_data = discharge.append(charge, ignore_index=True)

        clean_data = clean_data.reset_index(drop=True)

        clean_cycle = {name: clean_data}
        d.update(clean_cycle)
        # print("adding file to dictionary" + str(count) + ' ' + str(name))
    for key in d:
        clean_cycle_df = d[key]
        cyclename = key
        writer = ExcelWriter(save_filepath + cyclename + 'Clean' + '.xlsx')
        clean_cycle_df.to_excel(writer)
        writer.save()
    print('All cycles cleaned and saved in folder "' + save_filepath + '" .')
    return


def get_clean_sets(import_filepath, save_filepath):
    """Imports all clean cycles of data from import path and appends
    them into complete sets of battery data, saved into save_filepath"""
    assert os.path.exists(import_filepath) == 1
    assert os.path.exists(save_filepath) == 1
    rootdir = import_filepath
    file_list = [f for f in glob.glob(os.path.join(rootdir, '*.xlsx'))]
    # iterate through dir to get excel files
    d = {}  # initiate dict for data storage
    count = 0
    list_bats = []
    for file in file_list:
        count += 1
        name = os.path.split(file)[1].split('.')[0]
        batname = name.split('-')[0]
        if batname not in list_bats:
            list_bats.append(batname)
        else:
            None

    set_dict = {}

    for i in range(len(list_bats)):
        batID = list_bats[i]
        setdf = pd.DataFrame()
        for file in file_list:
            name = os.path.split(file)[1].split('.')[0]
            batname = name.split('-')[0]
            if batname == batID:
                df = pd.read_excel(file)
                setdf = setdf.append(df, ignore_index=True)
            else:
                None
        setdf = setdf.sort_values(['Voltage(V)'], ascending=True)
        setdf = setdf.reset_index(drop=True)
        newset = {batID: setdf}
        set_dict.update(newset)

    for key, value in set_dict.items():
        writer = ExcelWriter(save_filepath + key + 'CleanSet'+'.xlsx')
        value.to_excel(writer)
        writer.save()

    print('All clean cycles recombined and saved in folder "' +
          save_filepath + '" .')
    return
############################
# Component Functions
############################


def get_data(filepath):
    """Imports all data in given path"""
    assert type(filepath) == str, 'Input must be a string'
    rootdir = filepath
    # iterate through dir to get excel files
    file_list = [f for f in glob.glob(os.path.join(rootdir, '*.xlsx'))]

    d = {}
    # initiate dict for data storage
    count = 0
    for file in file_list:
        count += 1
        name = os.path.split(file)[1].split('.')[0]
        data = pd.read_excel(file, 1)
        new_set = {name: data}
        d.update(new_set)
    return d

# separate out dataframes into cycles


def sep_cycles(dataframe):
    """This function separates out the cycles in the battery dataframe by
     grouping by the 'Cycle_Index' column, and putting them in a dictionary."""
    assert type(dataframe) == pd.DataFrame, 'Input must be a dataframe'
    gb = dataframe.groupby(by=['Cycle_Index'])
    cycle_dict = dict(iter(gb))
    return cycle_dict


def save_sep_cycles_xlsx(cycle_dict, battname, path_to_folder):
    """This saves the separated out cycles into different excel files,
    beginning with the battery name. Battname and path to folder must
    be strings."""
    assert type(cycle_dict) == dict, 'First entry must be a dictionary'
    assert type(battname) == str, 'Second entry must be a string'
    assert type(path_to_folder) == str, 'Path to output must be a string'
    for i in range(1, len(cycle_dict)+1):
        cycle_dict[i]['Battery_Label'] = battname
    for i in range(1, len(cycle_dict)+1):
        writer = ExcelWriter(path_to_folder + battname +
                             '-'+'Cycle' + str(i) + '.xlsx')
        cycle_dict[i].to_excel(writer)
        writer.save()
    return


def calc_dv_dqdv(cycle_df):
    """This function calculates the dv and the dq/dv for a dataframe."""
    assert type(cycle_df) == pd.DataFrame
    assert 'Voltage(V)' in cycle_df.columns
    assert 'Discharge_Capacity(Ah)' in cycle_df.columns
    assert 'Charge_Capacity(Ah)' in cycle_df.columns

    cycle_df = cycle_df.reset_index(drop=True)
    cycle_df['dV'] = None
    cycle_df['Discharge_dQ'] = None
    cycle_df['Charge_dQ'] = None
    cycle_df['Discharge_dQ/dV'] = None
    cycle_df['Charge_dQ/dV'] = None
    for i in range(1, len(cycle_df)):
        cycle_df.loc[i, ('dV')] = cycle_df.loc[i, ('Voltage(V)')
                                               ] - cycle_df.loc[i-1, ('Voltage(V)')]
        cycle_df.loc[i, ('Discharge_dQ')] = cycle_df.loc[i, ('Discharge_Capacity(Ah)')
                                                         ] - cycle_df.loc[i-1, ('Discharge_Capacity(Ah)')]
        cycle_df.loc[i, ('Charge_dQ')] = cycle_df.loc[i, ('Charge_Capacity(Ah)')
                                                      ] - cycle_df.loc[i-1, ('Charge_Capacity(Ah)')]
    cycle_df['Discharge_dQ/dV'] = cycle_df['Discharge_dQ']/cycle_df['dV']
    cycle_df['Charge_dQ/dV'] = cycle_df['Charge_dQ']/cycle_df['dV']
    return cycle_df


def drop_0_dv(cycle_df_dv):
    '''Drop rows where dv=0 (or about 0) in a dataframe that has
    already had dv calculated. Then recalculate dv and calculate dq/dv'''
    # this will clean up the data points around V = 4.2V
    # (since they are holding voltage at 4.2V for a while).
    assert 'dV' in cycle_df_dv.columns
    assert 'Current(A)' in cycle_df_dv.columns

    cycle_df_dv = cycle_df_dv.reset_index(drop=True)

    cycle_df_dv['dv_close_to_zero'] = None

    for i in range(1, len(cycle_df_dv)):
        if isclose(cycle_df_dv.loc[i, ('Current(A)')], 0, abs_tol=10**-3):
            cycle_df_dv = cycle_df_dv.drop(index=i)

    cycle_df_dv = cycle_df_dv.reset_index(drop=True)
    switch_cd_index = np.where(np.diff(np.sign(cycle_df_dv['Current(A)'])))
    for i in switch_cd_index:
        cycle_df_dv = cycle_df_dv.drop(cycle_df_dv.index[i+1])

    cycle_df_dv = cycle_df_dv.reset_index(drop=True)

    for i in range(1, len(cycle_df_dv)):
        if isclose(cycle_df_dv.loc[i, ('dV')], 0, abs_tol=10**-3):
            # was -3.5 before
            cycle_df_dv.loc[i, ('dv_close_to_zero')] = False
        else:
            cycle_df_dv.loc[i, ('dv_close_to_zero')] = True

    while (False in cycle_df_dv['dv_close_to_zero'].values or
           cycle_df_dv['dV'].max() > 0.7 or cycle_df_dv['dV'].min() < -0.7):

        cycle_df_dv = cycle_df_dv.reset_index(drop=True)

        for i in range(1, len(cycle_df_dv)):
            if isclose(cycle_df_dv.loc[i, ('dV')], 0, abs_tol=10**-3):
                cycle_df_dv = cycle_df_dv.drop(index=i)

        cycle_df_dv = cycle_df_dv.reset_index(drop=True)

        separate_dis_char = np.where(
            np.diff(np.sign(cycle_df_dv['Current(A)'])))

        for i in range(1, len(cycle_df_dv)):
            if (cycle_df_dv.loc[i, ('dV')] > 0.7 or cycle_df_dv.loc[i, ('dV')] < -0.7):
                cycle_df_dv = cycle_df_dv.drop(index=i)

        cycle_df_dv = cycle_df_dv.reset_index(drop=True)

        for i in range(1, len(cycle_df_dv)):
            cycle_df_dv.loc[i, ('dV')] = cycle_df_dv.loc[i,
                                                         ('Voltage(V)')] - cycle_df_dv.loc[i-1, ('Voltage(V)')]
            if isclose(cycle_df_dv.loc[i, ('dV')], 0, abs_tol=10**-3):
                cycle_df_dv.loc[i, ('dv_close_to_zero')] = False
            else:
                cycle_df_dv.loc[i, ('dv_close_to_zero')] = True

        cycle_df_dv = cycle_df_dv.reset_index(drop=True)

    cycle_df_dv = cycle_df_dv.reset_index(drop=True)

    # recalculating dv and dq's after dropping rows
    for i in range(1, len(cycle_df_dv)):
        cycle_df_dv.loc[i, ('dV')] = cycle_df_dv.loc[i,
                                                     ('Voltage(V)')] - cycle_df_dv.loc[i-1, ('Voltage(V)')]
        cycle_df_dv.loc[i, ('Discharge_dQ')] = cycle_df_dv.loc[i, (
            'Discharge_Capacity(Ah)')] - cycle_df_dv.loc[i-1, ('Discharge_Capacity(Ah)')]
        cycle_df_dv.loc[i, ('Charge_dQ')] = cycle_df_dv.loc[i, ('Charge_Capacity(Ah)')
                                                            ] - cycle_df_dv.loc[i-1, ('Charge_Capacity(Ah)')]
    # recalculate dq/dv
    cycle_df_dv['Discharge_dQ/dV'] = cycle_df_dv['Discharge_dQ'] / \
        cycle_df_dv['dV']
    cycle_df_dv['Charge_dQ/dV'] = cycle_df_dv['Charge_dQ']/cycle_df_dv['dV']
    cycle_df_dv = cycle_df_dv.dropna(subset=['Discharge_dQ/dV'])
    cycle_df_dv = cycle_df_dv.dropna(subset=['Charge_dQ/dV'])
    cycle_df_dv = cycle_df_dv.reset_index(drop=True)

    cycle_df_dv = cycle_df_dv.reset_index(drop=True)
    return cycle_df_dv


def sep_char_dis(df_dqdv):
    '''Takes a dataframe of one cycle with calculated dq/dv and
    separates into charge and discharge differential capacity curves'''
    assert 'Charge_dQ/dV' in df_dqdv.columns
    assert 'Discharge_dQ/dV' in df_dqdv.columns
    charge = df_dqdv[df_dqdv['Current(A)'] > 0]
    charge.is_copy = None
    charge = charge.reset_index(drop=True)
    charge['dQ/dV'] = charge['Charge_dQ/dV']

    for i in range(1, len(charge)):
        if charge.loc[i, ('dQ/dV')] == 0:
            charge = charge.drop(index=i)
    charge = charge.reset_index(drop=True)

    discharge = df_dqdv[df_dqdv['Current(A)'] < 0]
    discharge.is_copy = None
    discharge['dQ/dV'] = discharge['Discharge_dQ/dV']
    discharge = discharge.reset_index(drop=True)

    for i in range(1, len(discharge)):
        if discharge.loc[i, ('dQ/dV')] == 0:
            discharge = discharge.drop(index=i)
    discharge = discharge.reset_index(drop=True)

    return charge, discharge


def my_savgolay(dataframe, windowlength, polyorder):
    """Takes battery dataframe with a dQ/dV column and applies a
    sav_golay filter to it, returning the dataframe with a new
    column called Smoothed_dQ/dV"""
    assert not windowlength % 2 == 0
    # asserts this is an odd number - necessary to run the
    # savgol_filter function
    assert polyorder < windowlength
    # necessary to run the savgol_filter function as well.
    unfilt = pd.concat([dataframe['dQ/dV']])
    unfiltar = unfilt.values
    # converts into an array
    dataframe['Smoothed_dQ/dV'] = scipy.signal.savgol_filter(
        unfiltar, windowlength, polyorder)
    # had windowlength = 21 and polyorder = 3 before
    return dataframe



from __future__ import absolute_import, division, print_function
from os.path import join as pjoin

# Format expected by setup.py and doc/source/conf.py: string of form "X.Y.Z"
_version_major = 0
_version_minor = 1
_version_micro = ''  # use '' for first of series, number for 1 and above
_version_extra = 'dev'
# _version_extra = ''  # Uncomment this for full releases

# Construct full version string from these.
_ver = [_version_major, _version_minor]
if _version_micro:
    _ver.append(_version_micro)
if _version_extra:
    _ver.append(_version_extra)

__version__ = '.'.join(map(str, _ver))

CLASSIFIERS = ["Development Status :: In Developement",
               "Environment :: Console",
               "Intended Audience :: Researchers",
               "License :: OSI Approved :: MIT License",
               "Operating System :: OS Independent",
               "Programming Language :: Python",
               "Topic :: Batteries and Charged Chinchillas"]

# Description should be a one-liner:
description = "Chachies: a program to help understand battery data."
# Long description will go up on the pypi page
long_description = """
Chachies
========
Chachies is a program to visualize and classify the chemistry of
batteries using machine learning and a DASH console.
To get started using these components in your own software, please go to the
repository README_.
.. _README: https://github.com/tacohen125/chachies/blob/master/README.md
License
=======
``chachies`` is licensed under the terms of the MIT license. See the file
"LICENSE" for information on the history of this software, terms & conditions
for usage, and a DISCLAIMER OF ALL WARRANTIES.
All trademarks referenced herein are property of their respective holders.
Copyright (c) 2018--, Nicole Thompson, Sarah Alamdari, Robert Masse,
Theodore Cohen
"""

NAME = "chachies"
MAINTAINER = "Nicole Thompson"
MAINTAINER_EMAIL = "nicolet5@uw.edu"
DESCRIPTION = description
LONG_DESCRIPTION = long_description
URL = "https://github.com/tacohen125/chachies"
DOWNLOAD_URL = ""
LICENSE = "MIT"
AUTHOR = "Nicole Thompson, Sarah Alamdari, Robert Masse, Theodore Cohen"
AUTHOR_EMAIL = "nicolet5@uw.edu"
PLATFORMS = "OS Independent"
MAJOR = _version_major
MINOR = _version_minor
MICRO = _version_micro
VERSION = __version__
PACKAGE_DATA = {'chachies': [pjoin('data', '*')]}
REQUIRES = ["Python3, LMFit, numpy, pandas, peakUtils, scikit-learn," +
			"scipy, sklearn, xlrd, DASH, plotly"]



import scipy.signal
import pandas as pd
import numpy as np
import peakutils
from lmfit import models
from chachies import chachifuncs as ccf
import os
import glob

################################
# OVERALL Wrapper Function
################################


def ML_generate(import_filepath):
    """Generates a dataframe containing charge and discharge
    descriptors/error parameters. Also writes descriptors to an
    excel spreadsheet 'describe.xlsx' import_filepath = filepath
    containing cleaned separated cycles"""

    # checks that the file exists
    assert os.path.exists(import_filepath), 'The file does not exist'

    # check that the whatever is passed to ML_generate is a string
    assert isinstance(import_filepath, str), 'The input should be a string'

    # creates dataframe of descriptors for the charge/discharge
    # cycles of all batteries
    df_ch = process.df_generate(import_filepath, 'c')
    df_dc = process.df_generate(import_filepath, 'd')
    # concats charge and discharge cycles
    df_final = pd.concat([df_ch, df_dc], axis=1)
    # drops any duplicate rows
    df_final = df_final.T.drop_duplicates().T
    # saves data to an excel file
    writer = pd.ExcelWriter('describe.xlsx')
    df_final.to_excel(writer, 'Sheet1')
    writer.save()

    return df_final

############################
# Sub - Wrapper Functions
############################
# data processing that calls from fitting class


class process:

    # first function called by ML_generate
    def df_generate(import_filepath, cd):
        """Creates a pandas dataframe for each battery's charge/
        discharge cycle in the import_filepath folder.
        import_filepath = filepath containing cleaned separated cycles
        cd = 'c' for charge and 'd' for discharge

        Output:
        df_ch = pandas dataframe for all cycles of all batteries in a
        col_ch = list of numbers of columns for each battery"""

        assert cd == (
            'c' or 'd'), 'This must be charge (c) or discharge (d) data'

        # generates a list of datafiles to analyze
        rootdir = import_filepath
        file_list = [f for f in glob.glob(os.path.join(rootdir, '*.xlsx'))]
        # iterate through dir to get excel file

        # generates a list of unique batteries
        list_bats = []

        for file in file_list:

            # splits file paths to get battery names
            name = os.path.split(file)[1].split('.')[0]
            batname = name.split('-')[0]

            # adds unique battery names to the list of batteries
            if batname not in list_bats:
                list_bats.append(batname)
            else:
                None

        # notifies user of successful import
        notice = 'Successfully extracted all battery names for ' + cd
        print(notice)

        # generates a blank dataframe of charge/discharge descriptors
        df_ch = process.pd_create(cd)

        # begins generating dataframe of descriptors
        name_ch = []
        for bat in list_bats:

            # notifies user which battery is being fit
            notice = 'Fitting battery: ' + bat + ' ' + cd
            print(notice)

            # generates dataframe of descriptor fits for each battery
            df = process.imp_all(import_filepath, bat, cd)

            # generates an iterative list of names for the 'name'
            # column of the final dataframe
            name_ch = name_ch + [bat] * len(df.index)

            # concats dataframe from current battery with previous
            # batteries
            df_ch = pd.concat([df_ch, df])

        # adds name column to the dataframe
        df_ch['names'] = name_ch

        return df_ch

    def imp_all(source, battery, cd):
        """Generates a Pandas dataframe of descriptors for a single battery

        source = string containing directory with the excel sheets
        for individual cycle data
        battery = string containing excel spreadsheet of file name
        cd = either 'c' for charge or 'd' for discharge

        Output:
        charge_descript = pandas dataframe of charge descriptors"""

        # check that the battery label is a string
        # check that the whatever is passed to ML_generate is a string
        # check that 'c' or 'd' is passed
        assert isinstance(source, str), 'The input should be a string'
        assert isinstance(battery, str), 'The input should be a string'
        assert cd == (
            'c' or 'd'), 'This must be charge (c) or discharge (d) data'

        # generates list of battery files for import
        file_pref = battery + '*.xlsx'
        file_list = [f for f in glob.glob(os.path.join(source, file_pref))]

        # sorts by cycle
        cycle = []

        # extracts cycle number from file name using known pattern
        for file in file_list:
            cyc1 = os.path.split(file)[1].split('Clean')[0]
            cyc = os.path.split(cyc1)[1].split('-Cycle')[1]
            cycle.append(int(cyc))

        # sorts cycle numbers
        cyc_sort = sorted(cycle)

        # determines order of indexes that will properly sort the data
        cyc_index = []
        for cyc in cyc_sort:
            cyc_index.append(cycle.index(cyc))

        # reindexes file list using the lists of indices from above
        file_sort = []
        for indices in cyc_index:
            file_sort.append(file_list[indices])

        # this is the end of the shit that sorts by cycle
        charge_descript = process.pd_create(cd)
        # iterates over the file list and the cycle number
        for file_val, cyc_loop in zip(file_sort, cyc_sort):

            # determines dictionary of descriptors from file data
            c = process.imp_one_cycle(file_val, cd, cyc_loop, battery)
            if c != 'throw':
                # generates list of dictionaries while rejecting any that
                # return the 'throw' error
                charge_descript = process.pd_update(charge_descript, c)

        return charge_descript

    def pd_create(cd):
        """Creates a blank dataframe containing either charge or
        discharge descriptors/error parameters

        cd = either 'c' for charge or 'd' for discharge

        Output:
        blank pandas dataframe with descriptor columns and cycle number rows"""

        # check that 'c' or 'd' is passed
        assert cd == (
            'c' or 'd'), 'This must be charge (c) or discharge (d) data'

        # number of descriptors it generates
        n_desc = 19

        # determines prefix string based on need for a charge or
        # discharge dataframe
        if cd == 'c':
            prefix = 'ch_'
        else:
            prefix = 'dc_'

        # generates list of names for the top of the descriptors dataframe
        names = []
        for ch in np.arange(n_desc):
            names.append(prefix + str(int(ch)))

        # adds names of error parameters to the end of the descriptor list
        names = names + [prefix+'AIC', prefix+'BIC', prefix+'red_chi_squared']

        # creates pandas dataframe with necessary heading
        # print(names)
        desc = pd.DataFrame(columns=names)

        return desc

    def pd_update(desc, charge_descript):
        """adds a list of charge descriptors to a pandas dataframe

        desc = dataframe from pd_create
        charge_descript = descriptor dictionaries

        Output:
        pandas dataframe with a row of descriptors appended on"""

        # check if the inputs have the right Type
        assert isinstance(
            desc, pd.core.frame.DataFrame), "This input must be a pandas dataframe"
        assert isinstance(
            charge_descript, dict), "Stop right there, only dictionaries are allowed in these parts"

        # converts the dictionary of descriptors into a list of descriptors
        desc_ls = process.dict_2_list(charge_descript)

        # adds zeros to the end of each descriptor list to create
        # a list with 22 entries
        # also appends error parameters to the end of the descriptor list
        desc_app = desc_ls + \
            np.zeros(19-len(desc_ls)).tolist() + charge_descript['errorParams']

        # generates a dataframe of descriptors
        desc_df = pd.DataFrame([desc_app], columns=desc.columns)
        # combines row of a dataframe with previous dataframe
        desc = pd.concat([desc, desc_df], ignore_index=True)

        return desc

    # used by pd_update
    def dict_2_list(desc):
        """Converts a dictionary of descriptors into a list for
        pandas assignment

        desc = dictionary containing descriptors

        Output:
        list of descriptors"""

        # this function is kinda pointless if you don't give it
        # a dictionary
        assert isinstance(
            desc, dict), "Stop right there, only dictionaries are allowed in these parts"

        # generates an initial list from the coefficients
        desc_ls = list(desc['coefficients'])

        # determines whether or not there are peaks in the datasent
        if 'peakSIGMA' in desc.keys():

            # iterates over the number of peaks
            for i in np.arange(len(desc['peakSIGMA'])):
                # appends peak descriptors to the list in order of peak number
                desc_ls.append(desc['peakLocation(V)'][i])
                desc_ls.append(desc['peakHeight(dQdV)'][i])
                desc_ls.append(desc['peakSIGMA'][i])
        else:
            pass

        return desc_ls

    def imp_one_cycle(file_val, cd, cyc_loop, battery):
        """imports and fits a single charge discharge cycle of a battery

        file_val = directory containing current cycle
        cd = either 'c' for charge or 'd' for discharge
        cyc_loop = cycle number
        battery = battery name

        output: a dictionary of descriptors for a single battery"""

        # make sure this is an Excel spreadsheet by checking the file extension
        assert file_val.split('.')[-1] == ('xlsx' or 'xls')

        # reads excel file into pandas
        testdf = pd.read_excel(file_val)

        # extracts charge and discharge from the dataset
        charge, discharge = ccf.sep_char_dis(testdf)

        # determines if the charge, discharge indicator was inputted correctly
        # assigns daframe for fitting accordingly
        if cd == 'c':
            df_run = charge
        elif cd == 'd':
            df_run = discharge
        else:
            raise TypeError(
                "Cycle type must be either 'c' for charge or 'd' for discharge.")

        # determines if a cycle should be passed into the descriptor
        # fitting function
        if (len(charge['Voltage(V)'].index) >= 10) and (len(discharge['Voltage(V)'].index) >= 10):
            # generates a dictionary of descriptors
            c = fitters.descriptor_func(
                df_run['Voltage(V)'], df_run['Smoothed_dQ/dV'], cd, cyc_loop, battery)
        # eliminates cycle number and notifies user of cycle removal
        else:
            notice = 'Cycle ' + str(cyc_loop) + ' in battery ' + battery + \
                ' had fewer than 10 datapoints and was removed from the dataset.'
            print(notice)
            c = 'throw'
        return c


class fitters:

    def descriptor_func(V_series, dQdV_series, cd, cyc, battery):
        """Generates dictionary of descriptors/error parameters

        V_series = Pandas series of voltage data
        dQdV_series = Pandas series of differential capacity data
        cd = either 'c' for charge and 'd' for discharge.

        Output:
        dictionary with keys 'codfficients', 'peakLocation(V)',
        'peakHeight(dQdV)', 'peakSIGMA', 'errorParams"""

        # make sure a single column of the data frame is passed to
        # the function
        assert isinstance(V_series, pd.core.series.Series)
        assert isinstance(dQdV_series, pd.core.series.Series)

        # appropriately reclassifies data from pandas to numpy
        sigx_bot, sigy_bot = fitters.cd_dataframe(V_series, dQdV_series, cd)

        # returns the indices of the peaks for the dataset
        i = fitters.peak_finder(V_series, dQdV_series, cd)

        # generates the necessary model parameters for the fit calculation
        par, mod = fitters.model_gen(
            V_series, dQdV_series, cd, i, cyc, battery)

        # returns a fitted lmfit model object from the parameters and data
        model = fitters.model_eval(V_series, dQdV_series, cd, par, mod)

        # initiates collection of coefficients
        coefficients = []

        for k in np.arange(4):
            # key calculation for coefficient collection
            coef = 'c' + str(k)
            # extracting coefficients from model object
            coefficients.append(model.best_values[coef])

        # creates a dictionary of coefficients
        desc = {'coefficients': coefficients}

        if len(i) > 0:
            # generates numpy array for peak calculation
            sigx, sigy = fitters.cd_dataframe(V_series, dQdV_series, cd)

            # determines peak location and height locations from raw data
            desc.update({'peakLocation(V)': sigx[i].tolist(
            ), 'peakHeight(dQdV)': sigy[i].tolist()})
            # initiates loop to extract
            sig = []
            for index in i:
                # determines appropriate string to call standard
                # deviation object from model
                center, sigma, amplitude, fraction, comb = fitters.label_gen(
                    index)
                sig.append(model.best_values[sigma])
        else:
            pass

            # updates dictionary with sigma key and object
            desc.update({'peakSIGMA': sig})

        # adds keys for the error parameters of each fit
        desc.update({'errorParams': [model.aic, model.bic, model.redchi]})

        return desc

    ############################
    # Sub - descriptor_func
    ############################

    def cd_dataframe(V_series, dQdV_series, cd):
        """Classifies and flips differential capactity data.

        V_series = Pandas series of voltage data
        dQdV_series = Pandas series of differential capacity data
        cd = either 'c' for charge and 'd' for discharge.

        Output:
        sigx = numpy array of signal x values
        sigy = numpy array of signal y values"""

        # converts voltage data to numpy array

        sigx = pd.to_numeric(V_series).as_matrix()

        # descriminates between charge and discharge cycle
        if cd == 'c':
            sigy = pd.to_numeric(dQdV_series).as_matrix()
        elif cd == 'd':
            sigy = -pd.to_numeric(dQdV_series).as_matrix()

        # check that the ouptut for these fuctions is positive
        # (with a little wiggle room of 0.5)
        threshold = -0.5
        min_sigy = np.min(sigy)
        assert min_sigy > threshold

        return sigx, sigy

    def peak_finder(V_series, dQdV_series, cd):
        """Determines the index of each peak in a dQdV curve

        V_series = Pandas series of voltage data
        dQdV_series = Pandas series of differential capacity data
        cd = either 'c' for charge and 'd' for discharge.

        Output:
        i = list of indexes for each found peak"""

        assert len(dQdV_series) > 10

        sigx, sigy = fitters.cd_dataframe(V_series, dQdV_series, cd)

        windowlength = 25
        if len(sigy) > windowlength:
            sigy_smooth = scipy.signal.savgol_filter(sigy, windowlength, 3)
        elif len(sigy) > 10:
            sigy_smooth = sigy

        i = peakutils.indexes(sigy_smooth, thres=.3 /
                              max(sigy_smooth), min_dist=9)

        return i

    def label_gen(index):
        """Generates label set for individual gaussian
        index = index of peak location

        output string format:
        'a' + index + "_" + parameter"""

        assert isinstance(index, (float, int))

        # generates unique parameter strings based on index of peak
        pref = str(int(index))
        comb = 'a' + pref + '_'

        cent = 'center'
        sig = 'sigma'
        amp = 'amplitude'
        fract = 'fraction'

        # creates final objects for use in model generation
        center = comb + cent
        sigma = comb + sig
        amplitude = comb + amp
        fraction = comb + fract

        assert isinstance((center, sigma, amplitude, fraction, comb), str)

        return center, sigma, amplitude, fraction, comb

    def model_gen(V_series, dQdV_series, cd, i, cyc, battery):
        """Develops initial model and parameters for battery data fitting.

        V_series = Pandas series of voltage data
        dQdV_series = Pandas series of differential capacity data
        cd = either 'c' for charge and 'd' for discharge.

        Output:
        par = lmfit parameters object
        mod = lmfit model object"""

        # generates numpy arrays for use in fitting
        sigx_bot, sigy_bot = fitters.cd_dataframe(V_series, dQdV_series, cd)

        # creates a polynomial fitting object
        mod = models.PolynomialModel(4)

        # sets polynomial parameters based on a
        # guess of a polynomial fit to the data with no peaks
        par = mod.guess(sigy_bot, x=sigx_bot)

        # prints a notice if no peaks are found
        if all(i) is False:
            notice = 'Cycle ' + str(cyc) + cd + \
                ' in battery ' + battery + ' has no peaks.'
            print(notice)

        # iterates over all peak indices
        else:
            for index in i:

                # generates unique parameter strings based on index of peak
                center, sigma, amplitude, fraction, comb = fitters.label_gen(
                    index)

                # generates a pseudo voigt fitting model
                gaus_loop = models.PseudoVoigtModel(prefix=comb)
                par.update(gaus_loop.make_params())

                # uses unique parameter strings to generate parameters
                # with initial guesses
                # in this model, the center of the peak is locked at the
                # peak location determined from PeakUtils

                par[center].set(sigx_bot[index], vary=False)
                par[sigma].set(0.01)
                par[amplitude].set(.05, min=0)
                par[fraction].set(.5, min=0, max=1)

                mod = mod + gaus_loop

        return par, mod

    def model_eval(V_series, dQdV_series, cd, par, mod):
        """evaluate lmfit model generated in model_gen function

        V_series = Pandas series of voltage data
        dQdV_series = Pandas series of differential capacity data
        cd = either 'c' for charge and 'd' for discharge.
        par = lmfit parameters object
        mod = lmfit model object

        output:
        model = lmfit model object fitted to dataset"""
        sigx_bot, sigy_bot = fitters.cd_dataframe(V_series, dQdV_series, cd)

        model = mod.fit(sigy_bot, par, x=sigx_bot)

        return model



from __future__ import absolute_import, division, print_function

__all__ = ["chachifuncs","descriptors","chachies", "version"]

from chachies import chachifuncs
from chachies import descriptors
from chachies.version import __version__



from chachies import chachifuncs as ccf
from chachies.descriptors import process 
from chachies.descriptors import fitters
import glob
import os
import pandas as pd
import pickle
from sklearn import linear_model
from sklearn import preprocessing
from sklearn import svm
from sklearn.model_selection import train_test_split
from sklearn.feature_selection import VarianceThreshold

# Import data
k_list = [f for f in glob.glob('data/K_descriptors/*.xlsx')]
c_list = [f for f in glob.glob('data/C_descriptors/*.xlsx')]

c_data = pd.DataFrame()
for each in c_list:
    df = pd.read_excel(each)
    c_data = c_data.append(df, ignore_index=True)
k_data = pd.DataFrame()
for each in k_list:
    df = pd.read_excel(each)
    k_data = k_data.append(df, ignore_index=True)
data = c_data.append(k_data)
data = data.T.drop_duplicates().T


# reset index and provide 0 1 classifiers
data = data.reset_index(drop=True)

for i in range(len(data)):
    if data.loc[i, ('names')].startswith('CS2_33'):
        data.loc[i, ('label')] = 'LiCoO2'
        data.loc[i, ('lasso')] = 0
    else:
        data.loc[i, ('label')] = 'LiFePO4'
        data.loc[i, ('lasso')] = 1

# split data
train, test = train_test_split(data, test_size=0.2, random_state=1010)


# choose data
train_y = train['lasso']  # what are we predicting
test_y = test['lasso']

train_x = train[['ch_5', 'ch_7', 'dc_5']]  # from LASSO
test_x = test[['ch_5', 'ch_7', 'dc_5']]

train_x_scaled = preprocessing.normalize(train_x, norm='l1')
test_x_scaled = preprocessing.normalize(test_x, norm='l1')

lin_svc = svm.LinearSVC().fit(train_x, train_y)
trainpred = lin_svc.predict(train_x_scaled)  # predict train data
testpred = lin_svc.predict(test_x_scaled)


filename = 'svc_model.sav'
pickle.dump(lin_svc, open(filename, 'wb'))


# load model from disk
class chachies_class:
    
    def clean(rootdir, path_to_raw_data_folder):
        '''Gets all raw data from the rootdir (ie 'data/') and specified folder
        (path_to_raw_data_folder), i.e. 'Source_Data' (which is within rootdir),
        and then:
        1. separates it into raw cycles and puts them in a folder
        (data/Separated_Cycles/)
        2. cleans those separated cycles and puts them in a folder
        (data/Clean_Separated_Cycles/)
        3. recombines the cleaned, separated cycles and saves those
        data sets in a folder (data/Clean_Whole_Sets/). These folders
        do not have to have existed previously.'''
        assert os.path.exists(rootdir) == 1
        if not os.path.exists(rootdir):
            print('The specified rootdir does not exist.')
        if not os.path.exists(rootdir+'Separated_Cycles/'):
            os.makedirs(rootdir+'Separated_Cycles/')
        if not os.path.exists(rootdir+'Clean_Separated_Cycles/'):
            os.makedirs(rootdir + 'Clean_Separated_Cycles/')
        if not os.path.exists(rootdir + 'Clean_Whole_Sets/'):
            os.makedirs(rootdir + 'Clean_Whole_Sets/')
        ccf.load_sep_cycles(rootdir + path_to_raw_data_folder, rootdir + 'Separated_Cycles/')
        ccf.get_clean_cycles(rootdir + 'Separated_Cycles/',rootdir + 'Clean_Separated_Cycles/')
        ccf.get_clean_sets(rootdir + 'Clean_Separated_Cycles/', rootdir+'Clean_Whole_Sets/')
        return

    def get_descriptors(import_filepath):
        """Generates a dataframe containing charge and discharge
        descriptors/error parameters. Also writes descriptors to an
        excel spreadsheet 'describe.xlsx' import_filepath = filepath
        containing cleaned separated cycles"""

        # checks that the file exists
        assert os.path.exists(import_filepath), 'The file does not exist'

        # check that the whatever is passed to ML_generate is a string
        assert isinstance(import_filepath, str), 'The input should be a string'

        # creates dataframe of descriptors for the charge/discharge
        # cycles of all batteries
        df_ch = process.df_generate(import_filepath, 'c')
        df_dc = process.df_generate(import_filepath, 'd')
        # concats charge and discharge cycles
        df_final = pd.concat([df_ch, df_dc], axis=1)
        #drops any duplicate rows
        df_final = df_final.T.drop_duplicates().T
        # saves data to an excel file
        writer = pd.ExcelWriter('describe.xlsx')
        df_final.to_excel(writer, 'Sheet1')
        writer.save()
        return df_final
    
    
    def classify(filename, test_x, test_y):
        """ Calls the pickled model and runs it on some set of data
        filename = svc_model.sav 
        test_x = contain 3 descriptors (first peak amplitude in charge cycle, first peak
        amplitude in discharge cycle, second peak location in charge cycle)
        test_y = battery type (or predicted battery type) """
        loaded_model = pickle.load(open(filename, 'rb'))
        predict = loaded_model.predict(test_x,test_y)
        result = loaded_model.score(test_x, test_y)
        return predict



from chachies import chachifuncs as ccf
import os
import pandas as pd

#### Functions to test:
# component functions: get_data, sep_cycles, save_sep_cycles_xlsx, calc_dv_dqdv, drop_0_dv, sep_char_dis, my_savgolay
# sub wrapper functions: load_sep_cycles, clean_calc_sep_smooth, get_clean_cycles, get_clean_sets
# overall function: get_all_data

#############################
# Component Functions Tests
#############################

def test_get_data():
	""" tests getting the data all from one folder path into a dictionary"""
	try:
		ccf.get_data(1)
		#should fail because input for get_data must be a string.
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by Asserts.')

def test_sep_cycles():
	""" tests function to separate cycles """
	#dataframe = pd.read_excel('test.xsls',1) #test should raise an error
	dataframe = [1,2,3]
	#test should pass if test generates an AssertionError
	try:
		ccf.sep_cycles(dataframe)
	except (AssertionError):
		pass
	else:
		raise Exception ("Exception not handled by Asserts")


def test_save_sep_cycles_xsls():
	"""tests function that saves separate cycles"""
	dataframe = pd.read_excel('data/test.xsls',1)
	cycle_dict = [dataframe,1,2]
	#should raise AssertionError. Inputs must be a dictionary, a battery name string, and a path as a string.
	try:
		ccf.save_sep_cycles_xlsx(cycle_dict, 'battery1', 'seperate_cycles/')
	except (AssertionError):
		pass
	else:
		raise Exception ("Exception not handled by Asserts")

def test_calc_dv_dqdv():
	"""Tests function that is calculating dv columns and dq/dv columns in the dataframe"""
	dataframe = pd.DataFrame({'col1': [1, 2, 3], 'col2': [4, 5, 6]})
	try:
		ccf.calc_dv_dqdv(dataframe)
		#should raise AssertionError because dataframe must have columns titled 'Voltage(V)', 'Charge_Capacity(Ah)', and 'Discharge_Capacity(Ah)'
	except (AssertionError):
		pass
	else:
		raise Exception ("Column labels exception not handled by asserts.")

def test_drop_0_dv():
	"""Tests that the dataframe put in has columns 'Current(A)' and 'dV'. This indicates the upstream function (calc_dv_dqdv) added the dv
	column correctly. """
	dataframe = pd.DataFrame({'dV': [0, 0.1, 0.2, 0.3], 'col2': [1, 3, 8, 3]})
	dataframe2 = pd.DataFrame({'Current(A)': [0, 0.1, 0.2, 0.3], 'col2': [1, 3, 8, 3]})
	try:
		ccf.drop_0_dv(dataframe)
		#should raise assertion error since the column 'Current(A)' doesn't exist in that dataframe
	except (AssertionError):
		pass
	else:
		raise Exception ("Column labels not containing 'Current(A)' exception not handled by asserts.")
	try:
		ccf.drop_0_dv(dataframe2)
	except (AssertionError):
		pass
	else:
		raise Exception ("Column labels not containing 'dV' exception not handled by asserts.")

def test_sep_char_dis():
	"""Tests that input is a dataframe with a column titled 'Discharge_dQ/dV' and a column titled 'Charge_dQ/dV', indicating upstream
	function drop_0_dv created those columns correctly."""
	dataframe = pd.DataFrame({'col1': [1, 2, 3], 'Discharge_dQ/dV': [3, 4, 5]})
	dataframe2 = pd.DataFrame({'col1': [1, 2, 3], 'Charge_dQ/dV': [3, 4, 5]})
	try:
		ccf.sep_char_dis(dataframe)
		#should raise an assertion error because Charge_dQ/dV doesn't exist in this dataframe.
	except (AssertionError):
		pass
	else:
		raise Exception ("Column labels not containing 'Charge_dQ/dV' exception not handled by asserts.")
	try:
		ccf.sep_char_dis(dataframe2)
	except (AssertionError):
		pass
	else:
		raise Exception ("Column labels not containing 'Discharge_dQ/dV' exception not handled by asserts.")

def test_my_savgolay():
	"""Tests the inputs of the my_savgolay function"""
	dataframe = pd.DataFrame({'dQ/dV': [1, 3, 4, 7, 3, 7, 21, 446, 32], 'col2': [21, 23, 42, 71, 38, 7, 21, 6, 2]})
	try:
		ccf.my_savgolay(dataframe, 4, 2)
		#should raise assertion because window length is not an odd number.
	except (AssertionError):
		pass
	else:
		raise Exception ("Windowlength must be odd exception not handled by asserts.")
	try:
		ccf.my_savgolay(dataframe, 5, 21)
		#should raise assertion because window length is less than the polyorder.
	except (AssertionError):
		pass
	else:
		raise Exception ("Polyorder must be less than windowlength exception not handled by asserts.")



##############################
# Sub Wrapper Functions Tests
##############################

def test_load_sep_cycles():
	""" Tests that the inputted file paths actually exist. (This function will NOT create a new directory if the paths don't exist -
	that is handled by the overall wrapper function.) """
	try:
		ccf.load_sep_cycles('NotARealFilePath/', 'AnotherNotARealFilePath/')
	except (AssertionError):
		pass
	else:
		raise Exception ('Not real file path exception not handled by assertion.')

def test_clean_calc_sep_smooth():
	""" Tests the input is a dataframe with columns with the correct names (dQ/dV, Current(A), Voltage(V))."""
	dataframe1 = pd.DataFrame({'dQ/dV': [1, 2, 4], 'col2:': [0, 2, 5], 'Voltage(V)': [6, 7, 8], 'Current(A)': [2, 3, 5], 'Discharge_Capacity(Ah)': [2, 1, 4], 'Charge_Capacity(Ah)': [1, 9, 14]})

	smooth_ch, smooth_dis = ccf.clean_calc_sep_smooth(dataframe1, 15, 3)
	assert 'Smoothed_dQ/dV' in smooth_ch.columns
	assert 'Smoothed_dQ/dV' in smooth_dis.columns
	assert not any(pd.isnull(smooth_ch['Smoothed_dQ/dV']))
	assert not any(pd.isnull(smooth_dis['Smoothed_dQ/dV']))
	#asserts there are no NaN values in the 'Smoothed_dQ/dV' column.

def test_get_clean_cycles():
	"""Tests the inputs for the function are real directory locations."""
	try:
		ccf.get_clean_cycles('NotARealFilePath/', 'AnotherNotARealFilePath/')
	except (AssertionError):
		pass
	else:
		raise Exception ('Not real file path exception not handled by asserts.')

def test_get_clean_sets():
	""" Tests the inputs for the function are real directory locations."""
	try:
		ccf.get_clean_cycles('NotARealFilePath/', 'AnotherNotARealFilePath/')
	except (AssertionError):
		pass
	else:
		raise Exception ('Not real file path exception not handled by asserts.')

########################
# Test Overall Wrapper
#######################

def test_get_all_data():
	""" Tests the inputs for get_all_data."""
	try:
		ccf.get_all_data('NotARealFilePath/', 'AnotherNotARealFilePath')
	except (AssertionError):
		pass
	else:
		raise Exception ('Note a real file path exception not handled by asserts.')



"""A setuptools-based setup module adapted from the Python Packaging Authority's
sample project.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    # Install this project using:
    # $ pip install chachies
    #
    # It lives on PyPI: https://pypi.org/project/chachies/
    name='chachies',

    # Versions should comply with PEP 440:
    # https://www.python.org/dev/peps/pep-0440/
    version='1.1',  # Required

    # This is a one-line description or tagline of what your project does. This
    # corresponds to the "Summary" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#summary
    description='Differential capacity plot visualization and machine learning model for battery chemistry classification',  # Required
    url='https://github.com/tacohen/chachies',
    author='The Charged Chinchillas',
    packages=find_packages(),


    install_requires=[
        'certifi==2018.1.18',
        'chardet==3.0.4',
        'idna==2.6',
        'lmfit==0.9.8',
        'numpy==1.14.2',
        'pandas==0.22.0',
        'PeakUtils==1.1.1',
        'python-dateutil==2.7.0',
        'pytz==2018.3',
        'requests==2.18.4',
        'scikit-learn==0.19.1',
        'scipy==1.0.0',
        'six==1.11.0',
        'sklearn==0.0',
        'urllib3==1.22',
        'xlrd==1.1.0',
        'dash==0.21.0',  # The core dash backend
        'dash-renderer==0.11.3',  # The dash front-end
        'dash-html-components==0.9.0',  # HTML components
        'dash-core-components==0.18.1',  # Supercharged components
        'plotly==2.4.1',
    ]

)



from chachies import chachifuncs as ccf
from chachies import descriptors
import glob
import matplotlib.pyplot as plt
import numpy as np
import os
import pandas as pd
import scipy.signal

a = 'data/testCaseData/7_24_13_1C_Cycle-Cycle11Clean.xlsx'
b = 'data/testCaseData/7_24_13_1C_Cycle-Cycle11Clean.xlsx'
c = 'data/testCaseData/7_24_13_1C_Cycle-Cycle11Clean.xlsx'
d = 'data/testCaseData/CS2_33_10_04_10-Cycle8Clean.xlsx'
e = 'data/testCaseData/CS2_33_10_04_10-Cycle10Clean.xlsx'
f = 'data/testCaseData/CS2_33_12_23_10-Cycle3Clean.xlsx'
g = 'data/testCaseData/CS2_33_12_23_10-Cycle12Clean.xlsx'

def test_ML_generate_is_given_a_real_file():
	""" tests loading something that isn't Excel data"""
	try:
		descriptors.ML_generate('test.xsls')
		#should fail because this file doesn't exist in the test path
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return


def test_ML_generate_is_given_a_number_as_input():
	"""tests passing something that isn't a string"""
	try:
		descriptors.ML_generate(5)
		#should fail because 5 is not a string.
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return


def test_df_generate_is_fed_c_or_d():
	"""test passing something that isn't c or d (charge or discharge)"""
	try:
		descriptors.process.df_generate(a,'asdf')
		#should fail because asdf is not c or d!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return

def test_df_generate_spits_out_a_dataframe():
	#assert that df_generate spits out a pandas dataframe
	assert isinstance(descriptors.process.df_generate(b,'c'), pd.core.frame.DataFrame), 'This should be a pandas dataframe'

	return

def test_imp_all_behaves_well():
	"""test passing something that isn't a string or c or d (charge or discharge)"""
	try:
		descriptors.process.imp_all(a,"str",'asdf')
		#should fail because asdf is not c or d!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	try:
		descriptors.process.imp_all(3,"str",'c')
		#should fail because 3 is not a file!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	try:
		descriptors.process.imp_all(a,7,'c')
		#should fail because 7 is not a str for the battery name
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return

def test_pd_create():
	"""test passing something that isn't a string or c or d (charge or discharge)"""
	try:
		descriptors.process.pd_create('asdf')
		#should fail because asdf is not c or d!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return

def test_pd_update():
	"""test to make sure pd_update takes the appropriate inputs"""
	mydict = {1:2,3:4}
	mydf = pd.DataFrame([1,2,3])
	try:
		descriptors.process.pd_update(mydf,"not a dict")
		#should fail because "not a dict" is not a dict!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	try:
		descriptors.process.pd_update("not a dataframe",mydict)
		#should fail because "not a dict" is not a dict!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return

def test_dict_2_list():
	"""make sure this only takes dictionaries as input"""
	try:
		descriptors.process.dict_2_list("not a dict")
		#should fail since "not a dict" is not a dict!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')
	return


def test_imp_one_cycle():
	"""tests loading something that isn't Excel data"""
	try:
		descriptors.process.imp_one_cycle('test.xsls', 'c', 7, "bob's battery" )
		#should fail because this file doesn't exist in the test path
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return


def test_descriptor_func():
	"""test the input is a single column of Voltage or smoothed dQ/dV data"""
	df = pd.DataFrame({"Voltage(V)":[1,2,3], "Smoothed_dQ/dV": [4,5,6]})
	proper_V = df['Voltage(V)']
	proper_dqdv = df["Smoothed_dQ/dV"]

	try:
		descriptors.fitters.descriptor_func(df, proper_dqdv,'c',7,"bob's batteries")
		#should raise assertion error since the column for voltage is bigger has more than one column
	except (AssertionError):
		pass
	else:
		raise Exception ("exception not handled by asserts.")
	try:
		descriptors.fitters.descriptor_func(proper_V, df,'c',7,"bob's batteries")
	except (AssertionError):
		pass
	else:
		raise Exception ("exception not handled by asserts.")

	return


def test_cd_dataframe():
	df = pd.DataFrame({"Voltage(V)":[1,2,3], "Smoothed_dQ/dV": [4,5,6]})
	proper_V = df['Voltage(V)']
	proper_dqdv = df["Smoothed_dQ/dV"]
	try:
		descriptors.fitters.cd_dataframe(proper_V,proper_dqdv, 'd')
		#this should fail because if dqdv is negative, it is probably discharge data and we want to flip to be positive for peak finding
	except (AssertionError):
		pass
	else:
		raise Exception ("Exception not handled by asserts")

	return


def test_peak_finder():
	"""test the input is a single column of Voltage or smoothed dQ/dV data"""
	df = pd.DataFrame({"Voltage(V)":[1,2,3], "Smoothed_dQ/dV": [4,5,6]})
	proper_V = df['Voltage(V)']
	proper_dqdv = df["Smoothed_dQ/dV"]
	try:
		descriptors.fitters.peak_finder(df, proper_dqdv,'c')
		#should raise assertion error since the dataset is too small to fit peaks
	except (AssertionError):
		pass
	else:
		raise Exception ("exception not handled by asserts.")

	return


def test_label_gen():
	"""make sure a number is the input and a string is the output"""
	try:
		descriptors.fitters.label_gen("not a number")
		#should fail because this only works with numbers
	except (AssertionError):
		pass
	else:
			raise Exception("Exception not handled by asserts")
	return

def test_imp_all_behaves_well():
	"""test passing something that isn't a string or c or d (charge or discharge)"""
	try:
		descriptors.process.imp_all(a,"str",'asdf')
		#should fail because asdf is not c or d!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	try:
		descriptors.process.imp_all(3,"str",'c')
		#should fail because 3 is not a file!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	try:
		descriptors.process.imp_all(a,7,'c')
		#should fail because 7 is not a str for the battery name
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return

def test_pd_create():
	"""test passing something that isn't a string or c or d (charge or discharge)"""
	try:
		descriptors.process.pd_create('asdf')
		#should fail because asdf is not c or d!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return

def test_pd_update():
	"""test to make sure pd_update takes the appropriate inputs"""
	mydict = {1:2,3:4}
	mydf = pd.DataFrame([1,2,3])
	try:
		descriptors.process.pd_update(mydf,"not a dict")
		#should fail because "not a dict" is not a dict!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	try:
		descriptors.process.pd_update("not a dataframe",mydict)
		#should fail because "not a dict" is not a dict!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return

def test_dict_2_list():
	"""make sure this only takes dictionaries as input"""
	try:
		descriptors.process.dict_2_list("not a dict")
		#should fail since "not a dict" is not a dict!
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')
	return


def test_imp_one_cycle():
	"""tests loading something that isn't Excel data"""
	try:
		descriptors.process.imp_one_cycle('test.xsls', 'c', 7, "bob's battery" )
		#should fail because this file doesn't exist in the test path
	except (AssertionError):
		pass
	else:
		raise Exception ('Exception not handled by asserts')

	return


def test_descriptor_func():
	"""test the input is a single column of Voltage or smoothed dQ/dV data"""
	df = pd.DataFrame({"Voltage(V)":[1,2,3], "Smoothed_dQ/dV": [4,5,6]})
	proper_V = df['Voltage(V)']
	proper_dqdv = df["Smoothed_dQ/dV"]

	try:
		descriptors.fitters.descriptor_func(df, proper_dqdv,'c',7,"bob's batteries")
		#should raise assertion error since the column for voltage is bigger has more than one column
	except (AssertionError):
		pass
	else:
		raise Exception ("exception not handled by asserts.")
	try:
		descriptors.fitters.descriptor_func(proper_V, df,'c',7,"bob's batteries")
	except (AssertionError):
		pass
	else:
		raise Exception ("exception not handled by asserts.")

	return


def test_cd_dataframe():
	df = pd.DataFrame({"Voltage(V)":[1,2,3], "Smoothed_dQ/dV": [4,5,6]})
	proper_V = df['Voltage(V)']
	proper_dqdv = df["Smoothed_dQ/dV"]
	try:
		descriptors.fitters.cd_dataframe(proper_V,proper_dqdv, 'd')
		#this should fail because if dqdv is negative, it is probably discharge data and we want to flip to be positive for peak finding
	except (AssertionError):
		pass
	else:
		raise Exception ("Exception not handled by asserts")

	return


def test_peak_finder():
	"""test the input is a single column of Voltage or smoothed dQ/dV data"""
	df = pd.DataFrame({"Voltage(V)":[1,2,3], "Smoothed_dQ/dV": [4,5,6]})
	proper_V = df['Voltage(V)']
	proper_dqdv = df["Smoothed_dQ/dV"]
	try:
		descriptors.fitters.peak_finder(df, proper_dqdv,'c')
		#should raise assertion error since the dataset is too small to fit peaks
	except (AssertionError):
		pass
	else:
		raise Exception ("exception not handled by asserts.")

	return


def test_label_gen():
	"""make sure a number is the input and a string is the output"""
	try:
		descriptors.fitters.label_gen("not a number")
		#should fail because this only works with numbers
	except (AssertionError):
		pass
	else:
			raise Exception("Exception not handled by asserts")

	return
